################################################################################
# This code fits a Bayesian hierarchical model to metabolic risk factor data   #
# using MCMC. The model includes linear and non-linear components, an age      #
# model and a study-coverage covariate model. The hierarchy has four levels:   #
# country, region, 'superregion' and world, which allows borrowing of strength #
# from countries and years with data to inform estimates for countries and     #
# years without data.                                                          #
#                                                                              #
# This version builds on code used to fit the model to mean BMI data to        #
# estimate national, regional and global BMI trends for NCD Risk Factor        #
# Collaboration "Trends in adult body-mass index in 200 countries from 1975 to # 
# 2014: a pooled analysis of 1698 population-based measurement studies with    #
# 19.2 million participants", Lancet 2016 (available for download at           #
# www.ncdrisc.org). Full details are provided in Finucane et al "Bayesian      # 
# estimation of population-level trends in measures of health status",         #
# Statistical Science 2014, and in the appendix of Danaei et al "National,     #
# regional, and global trends in systolic blood pressure since 1980:           #
# systematic analysis of health examination surveys and epidemiological        #
# studies with 786 country-years and 5.4 million participants", Lancet 2011.   #
#                                                                              #
# Here, the original model has been modify to be able to account for the       #
# non-linear growth patterns in 5-19 years of age, particularly during the     #
# adolescence period.                                                          #
################################################################################


################################################################################
# We have chosen to use a line width of more than 80 characters for some of    #
# the code, so that the whole of particular sections of the code can be seen   #
# at the same time. On our laptops and desktops the code displays most clearly #
# in Windows using Programmer's Notepad, and in Linux using gedit.             #
################################################################################

##### LIBRARIES AND FUNCTIONS ##################################################
##### Users should install these libraries beforehand                          #
################################################################################
library(Runuran)
library(TTR)
library(spam)
spam.options(cholsymmetrycheck=FALSE,structurebased=FALSE)
library(zoo)
source("NCDRisC_child_and_adolescent_height_and_bmi_functions.R")                       # Load functions
setwd(".")

##### USER DEFINED INPUTS ######################################################
##### Users should set appropriate values for the objects below                #
################################################################################
seedVal             <- 1                                                                # Assign each chain a unique seed value
sex                 <- "male"                                                           # Set gender: must be either "male" or "female"
age.min             <- 5                                                                # Set minimum age value
age.max             <- 19                                                               # Set maximum age value
max.dob             <- 2014                                                             # Set maximum year of birth
min.dob             <- 1966                                                             # Set minimum year of birth (1966 for height models and 1980 for BMI models)
nIts                <- 7000                                                             # Set number of MCMC iterations after thinning and including burn-in period
nThin               <- 10                                                               # Set thinning factor
epsilon             <- 0.001                                                            # Set hyperprior precision for a_g, b_g, alpha, beta, gamma, psi
height              <- TRUE                                                             # Set to FALSE to run the BMI model


##### PRELIMINARY CODE #########################################################
################################################################################
sex.val             <- switch(sex, male = 1, female = 2)                                # Set a numerical variable for sex: 1 for male and 2 for female
max.year            <- max.dob + 5                                                      # Maximum data year
min.year            <- min.dob + 5                                                      # Minimum data year
nLong               <- nIts * nThin                                                     # Total number of iterations in MCMC loop
nBurn               <- (nIts - 5000) * nThin                                            # Number of burn-in iterations to be removed at the end of the MCMC loop

# Age model inputs
if(height) {                                                                            # Parameters for height model
    average.value   <- 120                                                              # Centring value for height model
    by_cohort       <- TRUE                                                             # by_cohort = TRUE means that the time-scale is year of birth (i.e. cohort)
    if (sex == "female") {                                                              # Age model knot positions for girls
        knot1       <- 8
        knot2       <- 10
        knot3       <- 12
        knot4       <- 14
    } else {                                                                            # Age model knot positions for boys
        knot1       <- 10
        knot2       <- 12
        knot3       <- 14
        knot4       <- 16
    }
} else {                                                                                # Parameters for BMI model
    average.value   <- 25                                                               # Centring value for BMI model
    by_cohort       <- FALSE                                                            # by_cohort = FALSE means that the time-scale is data year
    knot1           <- 10                                                               # Age model knot 1 (for both boys and girls)
    knot2           <- 15                                                               # Age model knot 2 (sfor both boys and girls)
    knot3           <- NULL                                                             # Only two knots are used in BMI age model
    knot4           <- NULL                                                             # Only two knots are used in BMI age model
}
if (sex == "male") {
    middle.age      <- 14                                                               # Middle age used to centre age values in age model for boys
} else {
    middle.age      <- 11                                                               # Middle age used to centre age values in age model for girls
}
knot_list           <- sort(c(knot1, knot2, knot3, knot4))                              # List of knots used in age model (NULL values get excluded)
nk                  <- length(knot_list)                                                # Number of knots used in age model
ng                  <- nk + 3                                                           # Number of gamma parameters in age model
knot_name           <- paste("k", paste(knot_list, collapse = "_"), sep = "_")          # Collapsed string of knots used for file name

# Name of the output file
if(height) {
    filename        <- paste("Height", sex, knot_name, 'seed', seedVal, sep="_")        # File name for height model
} else {
    filename        <- paste("BMI", sex, knot_name, 'seed', seedVal, sep="_")           # File name for BMI model
}


##### DATA INPUT ###############################################################
##### The data are age- & sex-stratified summary statistics for each study.    #
#####                                                                          #
##### The data file must be a csv file with the following columns:             #
##### id_study: alphanumeric string (we suggest using only A-Z, 0-9 and _).    #
##### We suggest "iso3 code"_"data collection mid-year"_"survey name"          #
##### DATA CHARACTERISTICS                                                     #
##### sex: must be either "male" or "female", as the two sexes are modelled    #
##### separately                                                               #
##### age: positive number >=18 for adult-only analyses; 'age' is the mid-age  #
##### of the age group for this datapoint                                      #
##### mean_bmi or mean_height: positive number                                 #
##### se_bmi or se_height: positive number                                     #
##### STUDY CHARACTERISTICS
##### mid_year: integer year, as required by the non-linear component of the   #
##### model                                                                    #
##### survey_type: "National", "Subnational" or "Community"; used in the       #
##### analysis to account for possible bias in non-national studies            #
##### urban_rural: "urban", "rural" or "both"; used in the analysis to account #
##### for possible bias in urban-only or rural-only studies                    #
##### Country: country names                                                   #
##### Region: region names                                                     #
##### Superregion: superregion names                                           #

##### A metadata country file is required. This file must be a .csv file with  #
##### including all the countries for which the model will provide estimates.  #
##### The file should contain the following columns: Country, iso, Region, and #
##### Superregion

##### An additional covariate file including the percentage of urbanization    #
##### per country, region, and data year is also required. This file should be #
##### an .csv file, with the following columns: country, region, data_year, and#
##### perurb ('percentage of urbanization').                                   #
# Load covariate data
covariate.list      <- c("perurb")                                                      # Load covariate dataset
covar <- read.csv("covariates_2019_wUrbanisation.csv", header = TRUE)
covar               <- covar[order(covar$region,covar$country,covar$data_year),]        # Sort covariate dataset by region, country and year
covar$Z.weighted    <- covar$perurb
if(height) {
    start.year      <- min.dob
    end.year        <- max.dob
} else {
    start.year      <- min.dob + 5
    end.year        <- max.dob + 5
}
covar               <- subset(covar, data_year >= start.year & data_year <= end.year) 

# Load country metadata
data.country      <- read.csv("country_list2015.csv")

# Load height or bmi data
if (height == TRUE) {                                                                   # Height data
    data            <- read.csv("height_data.csv", stringsAsFactors = FALSE)            # Load dataset
    data            <- subset(data, !is.na(data$mean_height) & !is.na(data$se_height) & # Remove studies without mean_height or se_height; 
                                     mid_year <= max.year)                              # as well as studies with mid_year larger than max_year
    data            <- data[order(data$Superregion,data$Region,data$Country,data$mid_year), ] 
    data$DOB        <- data$mid_year - trunc(data$age)                                  # Define data of birth (DOB) variable
    subset          <- subset(data, data$sex==sex.val & age >= age.min & age <= 30 & 
                                  DOB >= min.dob & DOB <= max.dob)                      # Subset based on gender, DOB, age.min and age.max (30 for height models)
    subset$age      <- ifelse(subset$age > 19, 19, subset$age)                          # Use height data at age 20-30 years as proxy for height at age 19 years
    subset          <- subset[, c("iso", "id_study", "sex", "age",
                                  "N_height", "mean_height", "se_height",
                                  "mid_year", "DOB", "survey_type", "urban_rural",
                                  "Country", "Region", "Superregion")]
    colnames(subset)<- c("iso", "uid", "sex.final", "age",                              # Rename columns
                         "N_height", "mean", "sem",
                         "data_year", "cohort", "coverage", "scope",
                         "country", "region", "sregion")
} else {                                                                                # BMI data
    data            <- read.csv("bmi_data.csv", stringsAsFactors = FALSE)               # Load dataset
    data$mid_year   <- ifelse(data$mid_year >= (min.year - 3) & data$mid_year < min.year & data$survey_type == "National",
                              min.year, data$mid_year)                                  # Include national studies from up to 3 years before the minimum data year
    data            <- subset(data, !is.na(data$mean_bmi) & !is.na(data$se_bmi) & mid_year <= max.year) 
    data            <- data[order(data$Superregion,data$Region,data$Country,data$mid_year), ] 
    data$DOB        <- data$mid_year - trunc(data$age)
    subset          <- subset(data, data$sex==sex.val & age >= age.min &                # Subset based on gender, DOB, age.min and age.max
                                    age <= age.max & mid_year >= min.year & mid_year <= max.year)
    subset          <- subset[, c("iso", "id_study", "sex", "age",
                                  "N_bmi", "mean_bmi", "se_bmi", "mid_year", 
                                  "DOB", "survey_type", "urban_rural",
                                  "Country", "Region", "Superregion")]
    colnames(subset)<- c("iso", "uid", "sex.final", "age",                              # Rename columns
                         "N_bmi", "mean", "sem",
                         "data_year", "cohort", "coverage", "scope",
                         "country", "region", "sregion")
}
subset$region       <- as.character(subset$region)
subset$sregion      <- as.character(subset$sregion)
subset$region       <- as.character(subset$region)
subset$country      <- as.character(subset$country)
attach(subset)
uid                 <- paste(scope, data_year, uid, sep='')
subset$uid          <- uid
## Derived variables from urban/rural/mixed
rural               <- as.numeric(scope=="rural")   
urban               <- as.numeric(scope=="urban")   
mixed               <- as.numeric(scope=="both") 
## Age and cohort ##
if (by_cohort) {
    birth.cohort    <- cohort 
} else {
    birth.cohort    <- data_year
}
temp.med            <- trunc(mean(birth.cohort))
t                   <- (min(birth.cohort):max(birth.cohort)) - temp.med                 # Centred list of analysis years
temporal            <- (min(birth.cohort):max(birth.cohort))                            # List of analysis years
time                <- birth.cohort - temp.med                                          # Centred timepoints in data
age                 <- age - middle.age                                                 # Age centred by middle age
ageMat              <- cbind(age, age^2, age^3)                                         # Age matrix, mapping datapoints to ages
for(my_knot in knot_list) {                                                             # Age matrix using knots
    age_knot_tmp <- age + middle.age - my_knot; age_knot_tmp[age < -middle.age + my_knot] <- 0
    ageMat <- cbind(ageMat, age_knot_tmp^3)
}
ageMat.prime        <- t(ageMat)                                                        # Transposed age matrix
## Centre the outcome ##
y                   <- mean - average.value                                             # Centred outcome
## Set constants ##  
I                   <- length(mean)                                                     # Number of survey-year-sex-age datapoints               
J                   <- nlevels(data.country$Country)                                    # Number of countries for which estimates are made (as opposed to number of countries in the dataset) 
K                   <- nlevels(data.country$Region)                                     # Number of regions for which estimates are made
L                   <- nlevels(data.country$Superregion)                                # Number of superregions for which estimates are made    
N                   <- length(unique(uid))                                              # Number of studies in the data      
T                   <- length(t)                                                        # Length of analysis period


##### RANDOM WALK CONSTANTS ####################################################
##### Penalty matrix ###########################################################
##### See Rue and Held, p.110, and Danaei et al Webappendix, p.6                                                                  
P <- matrix(0, T, T)            
    P[1, (1:3)]     <- c(1, -2, 1)
    P[2, (1:4)]     <- c(-2, 5, -4, 1)
    for(i in 3:(T-2)) P[i, ((i-2):(i+2))] <- c(1, -4, 6, -4, 1)
    P[T-1, (T-3):T] <- c(1, -4, 5, -2)
    P[T, (T-2):T]   <- c(1, -2, 1)
##### Eigenanalysis for countries with no data #################################
##### Random walk with no data - see Danaei et al Webappendix p.15                          
eigenNoData             <- eigen(P)               
eigenNoData$values[T-1] <- eigenNoData$values[T] <- Inf   
EigenValGenInv          <- eigenNoData$val
EigenValGenInv[(T-1):T] <- 0     
SigmaGenInvNoTheta      <- eigenNoData$vec %*% (EigenValGenInv * t(eigenNoData$vec)) 
##### Constraints for random walk ##############################################
##### See Danaei et al Webappendix p.15
##### Allows identifiabilty of linear random intercepts and slopes              
A                   <- matrix(1, 2, T)
A[2,]               <- t - mean(t)


##### MAPPING MATRICES AND VECTORS #############################################
################################################################################
##### Matrices with lists of study IDs, countries, regions and all possible    #
##### combinations of country & year, region & year and superregion & year for #
##### which estimates are made                                                 #
################################################################################
uid.match           <- data.frame(name=unique(uid)[order(unique(uid))],
                                  number=1:length(unique(uid)))                         # Study country.lists from 1 to N
which.uid           <- match(uid,uid.match$name)                                        # Map data to study. Vector of length I, with values 1 to N
uid.match$coverage  <- coverage[match(uid.match$name,uid)]                              # Coverage of each study
country.match       <- data.frame(name=levels(data.country$Country), number=1:J)        # Name and country.list number of each country (1 to J)
region.match        <- data.frame(name=levels(data.country$Region), number=1:K)         # dim K*2: name region, id region (1 to K)
sregion.match       <- data.frame(name=levels(data.country$Superregion), number=1:L)    # dim L*2: name sregion, id sregion (1 to L)
country.time.match  <- data.frame(country=rep(levels(data.country$Country),each=T), 
                                  year=as.numeric(rep(temporal,J)), number = 1:(J*T))   # Country and year id (1 to J*T)
region.time.match   <- data.frame(region=rep(levels(data.country$Region),each=T), 
                                  year=as.numeric(rep(temporal,K)), number = 1:(K*T))   # Region and year id (1 to K*T)
sregion.time.match  <- data.frame(sregion=rep(levels(data.country$Superregion),each=T),
                                  year=as.numeric(rep(temporal,L)), number = 1:(L*T))   # Super-region and year id (1 to L*T)
which.country       <- match(country,country.match$name)                                # Map data to country. Vector of length I, with values from 1 to J
which.region        <- match(region,region.match$name)                                  # Map data to region. Vector of length I, with values from 1 to K
which.sregion       <- match(sregion,sregion.match$name)                                # Map data to super-region. Vector of length I, with values from 1 to L
which.time          <- time-min(time)+1                                                 # Map data to year. Vector of length I, with values from 1 to T
which.countryTime   <- match(paste(country,birth.cohort),
                             paste(country.time.match$country,country.time.match$year)) # Map data to country-year
which.countryTime.covar <- match(paste(country,birth.cohort),
                                 paste(covar$country, covar$data_year))                 # Identifies position of each datapoint's country-year in covariates
which.regionTime    <- match(paste(region,birth.cohort),
                           paste(region.time.match$region,region.time.match$year))      # Map data to region-year
which.sregionTime   <- match(paste(sregion,birth.cohort),
                            paste(sregion.time.match$sregion,sregion.time.match$year))  # Map data to super-region-year
multipleRegionsInSregion <- as.logical(!rowSums(table(region,sregion)[,(1:L)[colSums(table(region,sregion)>0)==1]])>0)  # Super-regions that are also regions. RE at super-region level
country.table       <- table(index(which.country),which.country)                        # Matrix 0/1 -> dim I * nb of countries with data
StudyToCountry      <- StudyToCountry.time <- matrix(0, I, J)
StudyToCountry[,as.numeric(colnames(country.table))] <- country.table[,as.numeric(index(colnames(country.table)))]         
StudyToCountry.time <- StudyToCountry * time                                                            
StudyToRegion       <- table(index(region),region)                                                       
StudyToRegion.time  <- table(index(region),region) * time                                           
StudyToSregion      <- table(index(sregion),sregion)                                                     
StudyToSregion.time <- table(index(sregion),sregion) * time
StudyToUid          <- table(index(which.uid),which.uid)                                                     
StudyToUid.time     <- table(index(which.uid),which.uid) * time
tPtsPerSregion      <- as.numeric(colSums(table(data_year,sregion)>0))                                   
tPtsPerRegion       <- as.numeric(colSums(table(data_year,region)>0))                                    
country             <- as.character(country)                                                                
country.match$name  <- as.character(country.match$name)
country.count       <- as.numeric(colSums(table(data_year,country)>0))                                
tPtsPerCountry      <- vector("integer", J)                                                              
tPtsPerCountry[as.numeric(colnames(country.table))] <- country.count[index(colnames(country.table))]
N_comm              <- sum(uid.match$coverage=="Community")
N_subn              <- sum(uid.match$coverage=="Subnational")
N_natl              <- sum(uid.match$coverage=="National")

##### Create "F" matrix ########################################################
##### This matrix is described in the appendix of Danaei et al, p.12           #
##### It maps each linear and nonlinear component, covariates and the study-   #
##### specific random effects to the corresponding parameters in the theta     #
##### matrix.                                                                  #
################################################################################
colnames(StudyToCountry) <- country.match$name
colnames(StudyToCountry.time) <- country.match$name
colnames(StudyToUid)<- uid.match$name
F                   <- cbind(rep(1, I),                                                 # Global intercept
                             time,                                                      # Global linear trend
                             StudyToSregion,                                            # Super-region intercepts
                             StudyToSregion.time,                                       # Super-region linear trends      
                             StudyToRegion[,multipleRegionsInSregion],                  # Region intercepts
                             StudyToRegion.time[,multipleRegionsInSregion],             # Region linear trends   
                             StudyToCountry,                                            # Country intercepts
                             StudyToCountry.time,                                       # Country linear trends   
                             StudyToUid,                                                # Study-specific random effects
                             as.numeric(coverage=="Subnational"),                       # Covariates
                             as.numeric(coverage=="Subnational")*time, 
                             as.numeric(coverage=="Community"),
                             as.numeric(coverage=="Community")*time,
                             covar[,"Z.weighted"][which.countryTime.covar],             # Column in F matrix corresponding to urbanisation offset
                             covar[,"Z.weighted"][which.countryTime.covar]*time,        # Column in F matrix corresponding to urbanisation slope offset
                             covar[,"Z.weighted"][which.countryTime.covar]*rural,       # Column in F matrix corresponding to rural-only offset
                             covar[,"Z.weighted"][which.countryTime.covar]*rural*time,  # Column in F matrix corresponding to rural-only slope offset
                             (1-covar[,"Z.weighted"][which.countryTime.covar])*urban,   # Column in F matrix corresponding to urban-only offset
                             (1-covar[,"Z.weighted"][which.countryTime.covar])*urban*time # Column in F matrix corresponding to urban-only slope offset
                            ) 
p                   <-  dim(F)[2] - (2+2*L+2*sum(multipleRegionsInSregion)+2*J+N)       # Number of parameters associated with covariates 
F.prime             <- t(F)
F.spam              <- as.spam(F)
F.spam.prime        <- t(F.spam)
F.age               <- F*age
F.spam.age          <- as.spam(F.age)
F.age.prime         <- t(F.age)
F.spam.age.prime    <- t(F.spam.age)


##### INITIAL VALUES ###########################################################
##### The following lines create empty vectors/matrices for model parameters   #
##### Parameter names in parentheses are used in Finucane et al & Danaei et al #
#####                                                                          #
##### NB. Those papers use 's' for subregion (equivalent to region here)       #
##### and 'r' for region (equivalent to superregion here)                      #
################################################################################
phi_natl            <- rep(NA, nIts)                                                    # Empty vector for log variance of ssre for national studies (log nu_"national")
phi_subn            <- rep(NA, nIts)                                                    # Empty vector for log variance of ssre for subnational studies (log nu_s)
phi_comm            <- rep(NA, nIts)                                                    # Empty vector for log variance of ssre for community studies (log nu_c)
tau                 <- rep(NA, nIts)                                                    # Empty vector for log variance for within-study errors that differ between age groups (log tau)
theta_r             <- rep(NA, nIts)                                                    # Empty vector for log precision parameter for random walk at region level (log lambda_s)
theta_s             <- rep(NA, nIts)                                                    # Empty vector for log precision parameter for random walk at superregion level (log lambda_r)
theta_g             <- rep(NA, nIts)                                                    # Empty vector for log precision parameter for random walk at global level (log lambda_g)
theta_c             <- rep(NA, nIts)                                                    # Empty vector for log precision parameter for random walk at country level (log lambda_c)
phi_s               <- rep(NA, nIts)                                                    # Empty vector for log variance for normal prior for superregion random intercepts (log kappa_a^r)
phi_r               <- rep(NA, nIts)                                                    # Empty vector for log variance for normal prior for region random intercepts (log kappa_a^s)
phi_c               <- rep(NA, nIts)                                                    # Empty vector for log variance for normal prior for country random intercepts (log kappa_a^c)
eta_s               <- rep(NA, nIts)                                                    # Empty vector for log variance for normal prior for superregion random slopes (log kappa_b^r)
eta_r               <- rep(NA, nIts)                                                    # Empty vector for log variance for normal prior for region random slopes (log kappa_b^s)
eta_c               <- rep(NA, nIts)                                                    # Empty vector for log variance for normal prior for country random slopes (log kappa_b^c)
deviance            <- rep(NA, nIts)                                                    # Empty vector for deviance
theta               <- matrix(NA, nIts, dim(F)[2])                                      # Empty matrix for theta matrix
w                   <- matrix(NA, nIts, T)                                              # Empty matrix for component of nonlinear trend at global level
sv                  <- matrix(NA, nIts, L*T)                                            # Empty matrix for component of nonlinear trend at superregional level
v                   <- matrix(NA, nIts, K*T)                                            # Empty matrix for component of nonlinear trend at regional level
u                   <- matrix(NA, nIts, J*T)                                            # Empty matrix for component of nonlinear trend at national level
gamma               <- matrix(NA, nIts, ng + ng*J)                                      # Empty matrix for age model parameters
sigma2              <- matrix(NA, nIts, ng)                                             # Empty matrix for variances for country-specific random spline coefficients

## The following lines set initial values for model parameters ##
theta_g.prop.sd     <-   0.25                                                           # Initial value for proposal SD for log precision parameter for random walk at global level (log lambda_g)
theta_s.prop.sd     <-   0.25                                                           # Initial value for proposal SD for log precision parameter for random walk at superregion level (log lambda_r)
theta_r.prop.sd     <-   0.25                                                           # Initial value for proposal SD for log precision parameter for random walk at region level (log lambda_s)
theta_c.prop.sd     <-   0.25                                                           # Initial value for proposal SD for log precision parameter for random walk at country level (log lambda_c)
phi_natl.prop.sd    <-   0.1                                                            # Initial value for proposal SD for log variance of random effects for national studies (log nu_"national")
phi_subn.prop.sd    <-   0.1                                                            # Initial value for proposal SD for log variance of random effects for subnational studies (log nu_"s")
phi_comm.prop.sd    <-   0.1                                                            # Initial value for proposal SD for log variance of random effects for community studies (log nu_"c")
tau.prop.sd         <-   0.1                                                            # Initial value for proposal SD for log variance for within-study errors that differ between age groups (log tau)
phi_c.prop.sd       <-   0.1                                                            # Initial value for proposal SD for log variance for normal prior for country random intercepts (log kappa_a^c)
phi_r.prop.sd       <-   0.1                                                            # Initial value for proposal SD for log variance for normal prior for region random intercepts (log kappa_a^s)
phi_s.prop.sd       <-   0.1                                                            # Initial value for proposal SD for log variance for normal prior for superregion random intercepts (log kappa_a^r)
eta_r.prop.sd       <-   0.3                                                            # Initial value for proposal SD for log variance for normal prior for country random slopes (log kappa_b^c)
eta_c.prop.sd       <-   0.3                                                            # Initial value for proposal SD for log variance for normal prior for region random slopes (log kappa_b^s)
eta_s.prop.sd       <-   0.3                                                            # Initial value for proposal SD for log variance for normal prior for superregion random slopes (log kappa_b^r)
log.sigma2.prop.sd  <- rep(0.5, ng)                                                     # Initial values for proposal SDs for log variances for country-specific random spline coefficients
phi_natl[1]         <-   -0.7                                                           # Initial value for log variance of random effects for national studies (log nu_"national")
phi_subn[1]         <-   0.75                                                           # Initial value for log variance of random effects for subnational studies (log nu_"s")
phi_comm[1]         <-   0.9                                                            # Initial value for log variance of random effects for community studies (log nu_"c")
tau[1]              <-   -1.4                                                           # Initial value for log variance for within-study errors that differ between age groups (log tau)
theta_g[1]          <-   10.1                                                           # Initial value for log precision parameter for random walk at global level (log lambda_g)
theta_s[1]          <-   9.8                                                            # Initial value for log precision parameter for random walk at superregion level (log lambda_r)
theta_r[1]          <-   9.5                                                            # Initial value for log precision parameter for random walk at region level (log lambda_s)
theta_c[1]          <-   9.1                                                            # Initial value for log precision parameter for random walk at country level (log lambda_c)
phi_c[1]            <-   1.55                                                           # Initial value for log variance for normal prior for country random intercepts (log kappa_a^c)
phi_r[1]            <-   1.75                                                           # Initial value for log variance for normal prior for region random intercepts (log kappa_a^s)
phi_s[1]            <-   2.25                                                           # Initial value for log variance for normal prior for superregion random intercepts (log kappa_a^r)
eta_c[1]            <-   -6.5                                                           # Initial value for log variance for normal prior for country random slopes (log kappa_b^c)
eta_r[1]            <-   -9.5                                                           # Initial value for log variance for normal prior for country region slopes (log kappa_b^s)
eta_s[1]            <-   -7.2                                                           # Initial value for log variance for normal prior for country superregion slopes (log kappa_b^r)
theta[1,]           <- rep(0, dim(F)[2])                                                # Initial values for theta matrix
w[1,]               <- rep(0, T)                                                        # Initial values for component of nonlinear trend at global level
sv[1,]              <- rep(0, L*T)                                                      # Initial values for component of nonlinear trend at superregional level
v[1,]               <- rep(0, K*T)                                                      # Initial values for component of nonlinear trend at regional level
u[1,]               <- rep(0, J*T)                                                      # Initial values for component of nonlinear trend at national level
gamma[1,]           <- rep(0, ng+ng*J)                                                  # Initial values for age model parameters
sigma2[1,]          <- rep(1e-10, ng)                                                   # Initial values for variances for country-specific random spline coefficients

## Overdisperse the starting values 
theta.max           <- 20                                                               # Constrain theta values for random walk to 20
phi_c[1]            <- rnorm(1, phi_c[1], phi_c.prop.sd/2.4*sqrt(6))       
phi_r[1]            <- rnorm(1, phi_r[1], phi_r.prop.sd/2.4*sqrt(6))       
phi_s[1]            <- rnorm(1, phi_s[1], phi_s.prop.sd/2.4*sqrt(6))       
eta_c[1]            <- rnorm(1, eta_c[1], eta_c.prop.sd/2.4*sqrt(6))       
eta_r[1]            <- rnorm(1, eta_r[1], eta_r.prop.sd/2.4*sqrt(6))       
eta_s[1]            <- rnorm(1, eta_s[1], eta_s.prop.sd/2.4*sqrt(6))       
tau[1]              <- rnorm(1, tau[1], tau.prop.sd/2.4)          
phi_natl[1]         <- rnorm(1, phi_natl[1], phi_natl.prop.sd/2.4*sqrt(3))
phi_subn[1]         <- ur(pinvd.new(udnorm(mean=phi_subn[1],
                                           sd=phi_subn.prop.sd/2.4*sqrt(3),lb=phi_natl[1])),1)
phi_comm[1]         <- ur(pinvd.new(udnorm(mean=phi_comm[1],
                                           sd=phi_comm.prop.sd/2.4*sqrt(3), lb=phi_subn[1])),1)
theta_c[1]          <- ur(pinvd.new(udnorm(mean=theta_c[1],sd=theta_c.prop.sd/2.4,
                                           ub=theta.max)),1)   
theta_r[1]          <- ur(pinvd.new(udnorm(mean=theta_r[1], sd=theta_r.prop.sd/2.4, 
                                           lb=theta_c[1],ub=theta.max)),1)
theta_s[1]          <- ur(pinvd.new(udnorm(mean=theta_s[1], sd=theta_s.prop.sd/2.4, 
                                           lb=theta_r[1],ub=theta.max)),1)
theta_g[1]          <- ur(pinvd.new(udnorm(mean=theta_g[1], sd=theta_g.prop.sd/2.4, 
                                           lb=theta_s[1],ub=theta.max)),1)
sigma2[1,]          <- exp(rnorm(ng, log(sigma2[1,]), log.sigma2.prop.sd/2.4)) 

## Diagonal of likelihood variance matrix ##
Sigma.diag          <- sem^2 + exp(tau[1])
SigmaInv.diag       <- 1/Sigma.diag

## Variance of study-specific random effects ##
V.ssre <- rep(NA, N)                                                                    # Empty vector for study-specific random effect variances
V.ssre[uid.match$coverage == "National"]    <- exp(phi_natl[1])                         # Set variances for random effects for national studies
V.ssre[uid.match$coverage == "Subnational"] <- exp(phi_subn[1])                         # Set variances for random effects for subnational studies
V.ssre[uid.match$coverage == "Community"]   <- exp(phi_comm[1])                         # Set variances for random effects for community studies

## Diagonal of Var(theta) matrix and invVar(gamma) matrix ##
V.diag              <- c(1/epsilon,                                                     # Flat variance for global intercept
                         1/epsilon,                                                     # Flat variance for global slope
                         rep(exp(phi_s[1]), L),                                         # Variances for normal prior for superregion random intercepts
                         rep(exp(eta_s[1]), L),                                         # Variances for normal prior for superregion random slopes
                         rep(exp(phi_r[1]), sum(multipleRegionsInSregion)),             # Variances for normal prior for region random intercepts
                         rep(exp(eta_r[1]), sum(multipleRegionsInSregion)),             # Variances for normal prior for region random slopes
                         rep(exp(phi_c[1]), J),                                         # Variances for normal prior for country random intercepts
                         rep(exp(eta_c[1]), J),                                         # Variances for normal prior for country random slopes
                         V.ssre)                                                        # Variances for study-specific random effects
V.diag              <- c(V.diag,rep(1/epsilon, p))                                      # Add flat variances for covariates
VInv.diag           <- 1/V.diag                                                         # Inverse variance
WInv.diag           <- c(rep(epsilon, ng), rep(1/sigma2[1,], each=J))                   # Inverse variance for age model

## Initial values for the theta update ##
Q.init              <- F.spam.prime %*% (SigmaInv.diag * F.spam)
Q.init              <- as.spam(Q.init)
diag(Q.init)        <- diag(Q.init) + VInv.diag
U.theta.init        <- chol(Q.init, cholsymmetrycheck=F)
varphiPlusC.age     <- rowSums((rep(1, I) %*% t(gamma[1,1:ng]) +  matrix(gamma[1,-(1:ng)], J, ng)[which.country,]) * ageMat)

## Initial values for the u/v/w update ##
V1uInv.diag         <- Vinv.M.u <- rep(0, J*T)
V1vInv.diag         <- Vinv.M.v <- rep(0, K*T)
V1svInv.diag        <- Vinv.M.sv<- rep(0, L*T)
V1wInv.diag         <- Vinv.M.w <- rep(0, T)

## Initial values for the gamma update ##
F.theta.Mm          <- y                                                                # Starting mean
R.age               <- ageMat                                                           # Starting matrix mapping ages to gamma parameters
for (i in 1:ncol(ageMat)) { 
    R.age <- cbind(R.age, StudyToCountry*ageMat[, i])
}
R.age.spam          <- as.spam(R.age)                                                   # Spam version of mapping matrix
R.age.spam.prime    <- t(R.age.spam)                                                    # Transposed spam version of mapping matrix
Q.init              <- R.age.spam.prime %*% (SigmaInv.diag * R.age.spam)                # Full conditional precision of gamma
diag(Q.init)        <- diag(Q.init) + WInv.diag                                         # Full conditional precision of gamma
U.gamma.init        <- chol(Q.init)                                                     # Cholesky decomposition
deviance[1]         <- LogLik(SigmaInv.diag,gamma[1,], R.age.spam, F.theta.Mm) * -2 + I * log(2*pi)

## Start chains at values consistent with the hyperparameter starting values ##
## Theta ######################################################################
theta[1,]            <- update.theta(SigmaInv.diag, VInv.diag, u[1,], v[1,], sv[1,], w[1,]) # First update of theta
F.theta.Mm           <- as.vector(F.spam %*% theta[1,] + u[1,which.countryTime] + v[1,which.regionTime] + sv[1,which.sregionTime] + w[1,which.time]) # Mean height/BMI at middle.age
R.age.spam           <- as.spam(R.age)                                                  # Update gamma mapping matrix based on new mean height/BMI at middle.age
R.age.spam.prime     <- t(R.age.spam)                                                   # Update gamma mapping matrix based on new mean height/BMI at middle.age
## Non-linear component at national level ######################################
#### Users should refer to pp5-7 and pp14-16 of Danaei et al for the line-by-  #
#### line details of this section of code ######################################
u.Prec.yMinusMean    <- SigmaInv.diag * (y - F %*% theta[1,] - (v[1,which.regionTime] + sv[1,which.sregionTime] + w[1,which.time]) - varphiPlusC.age) 
agg1                 <- aggregate(u.Prec.yMinusMean,list(which.countryTime),sum)
agg2                 <- aggregate(SigmaInv.diag,list(which.countryTime),sum)
Vinv.M.u[agg1[,1]]   <- agg1[,2]
V1uInv.diag[agg2[,1]]<- agg2[,2]
uStar                <- lapply(1:J, u.prop.function, tPtsPerCountry, V1uInv.diag, Vinv.M.u, theta_c[1], theta_c[1], u[1,])
u.star               <- rep(NA, J*T)
for(j in 1:J) { u.star[((j-1)*T+1):(j*T)] <- uStar[[j]]$u.star }
u[1,]                <- Re(u.star)           
F.theta.Mm           <- as.vector(F.spam %*% theta[1,] + u[1,which.countryTime] + v[1,which.regionTime] + sv[1,which.sregionTime] + w[1,which.time]) 
## Non-linear component at regional level ######################################
##### See Danaei et al pp5-7 and 14-16 for details #############################
v.Prec.yMinusMean    <- SigmaInv.diag  * (y - F %*% theta[1,] - (u[1,which.countryTime] + sv[1,which.sregionTime] + w[1,which.time]) - varphiPlusC.age) 
agg3                 <- aggregate(v.Prec.yMinusMean,list(which.regionTime),sum)
agg4                 <- aggregate(SigmaInv.diag,list(which.regionTime),sum)
Vinv.M.v[agg3[,1]]   <- agg3[,2]
V1vInv.diag[agg4[,1]]<- agg4[,2]
u.star               <- rep(NA, K*T)
uStar                <- lapply(1:K, u.prop.function, tPtsPerRegion, V1vInv.diag, Vinv.M.v, theta_r[1], theta_r[1], v[1,])
for(j in 1:K) { u.star[((j-1)*T+1):(j*T)] <- uStar[[j]]$u.star }
u.star[rep(!multipleRegionsInSregion, each=T)] <- 0
v[1,]                <- Re(u.star)
F.theta.Mm           <- as.vector(F.spam %*% theta[1,] + u[1,which.countryTime] + v[1,which.regionTime] + sv[1,which.sregionTime] + w[1,which.time]) 
## Non-linear component at superregional level #################################
##### See Danaei et al pp5-7 and 14-16 for details #############################
sv.Prec.yMinusMean    <- SigmaInv.diag  * (y - F %*% theta[1,] - (u[1,which.countryTime] + v[1,which.regionTime] + w[1,which.time]) - varphiPlusC.age) 
agg5                  <- aggregate(sv.Prec.yMinusMean,list(which.sregionTime),sum)
agg6                  <- aggregate(SigmaInv.diag,list(which.sregionTime),sum)
Vinv.M.sv[agg5[,1]]   <- agg5[,2]
V1svInv.diag[agg6[,1]]<- agg6[,2]
u.star                <- rep(NA, L*T)
uStar                 <- lapply(1:L, u.prop.function, tPtsPerSregion, V1svInv.diag, Vinv.M.sv, theta_s[1], theta_s[1], sv[1,])
for(j in 1:L) { u.star[((j-1)*T+1):(j*T)] <- uStar[[j]]$u.star }
u.star                <- Re(u.star)
sv[1,]                <- u.star
F.theta.Mm            <- as.vector(F.spam %*% theta[1,] + u[1,which.countryTime] + v[1,which.regionTime] + sv[1,which.sregionTime] + w[1,which.time]) 
## Non-linear component at global level ########################################
##### See Danaei et al pp5-7 and 14-16 for details #############################
w.Prec.yMinusMean     <- SigmaInv.diag  * (y - F %*% theta[1,] - (u[1,which.countryTime] + sv[1,which.sregionTime] + v[1,which.regionTime]) - varphiPlusC.age) 
agg7                  <- aggregate(w.Prec.yMinusMean,list(which.time),sum)
agg8                  <- aggregate(SigmaInv.diag,list(which.time),sum)
Vinv.M.w[agg7[,1]]    <- agg7[,2]
V1wInv.diag[agg8[,1]] <- agg8[,2]
uStar                 <- lapply(1, u.prop.function, 2, V1wInv.diag, Vinv.M.w, theta_g[1], theta_g[1], w[1,])
u.star                <- uStar[[1]]$u.star
w[1,]                 <- as.vector(Re(u.star))
F.theta.Mm            <- as.vector(F.spam %*% theta[1,] + u[1,which.countryTime] + v[1,which.regionTime] + sv[1,which.sregionTime] + w[1,which.time]) 
## Age model ##################################################################
Q                     <- R.age.spam.prime %*% (SigmaInv.diag * R.age.spam)                    # Full conditional precision of gamma
diag(Q)               <- diag(Q) + WInv.diag
U                     <- update.spam.chol.NgPeyton(U.gamma.init, Q)                           # Update Cholesky decomposition
VinvM                 <- as.matrix(R.age.spam.prime) %*% (SigmaInv.diag * (y - F.theta.Mm))   # Full conditional precision x full conditional mean
gamma[1,]             <- backsolve(U, forwardsolve(U, VinvM, trans=T) + rnorm(length(VinvM))) # Returns updated gamma following Gibbs step
varphiPlusC.age       <- rowSums((rep(1, I) %*% t(gamma[1,1:ng]) + matrix(gamma[1,-(1:ng)], J, ng)[which.country,]) * ageMat) # Update derived vector for future calculations 
deviance[1]           <- LogLik(SigmaInv.diag, gamma[1,], R.age, F.theta.Mm) * -2 + I * log(2*pi)

## Set current value holders ###############################################
V.acc <- theta_c.acc <- theta_r.acc <- theta_s.acc <- theta_g.acc <- tau.acc <- phi.acc <- 0
sigma2.acc            <-   rep(0, ng)
phi_natl.current      <-   phi_natl[1]      
phi_subn.current      <-   phi_subn[1]      
phi_comm.current      <-   phi_comm[1]      
tau.current           <-   tau[1]         
theta_g.current       <-   theta_g[1]      
theta_s.current       <-   theta_s[1]      
theta_r.current       <-   theta_r[1]      
theta_c.current       <-   theta_c[1]      
phi_c.current         <-   phi_c[1]      
phi_r.current         <-   phi_r[1]      
phi_s.current         <-   phi_s[1]      
eta_c.current         <-   eta_c[1]      
eta_r.current         <-   eta_r[1]      
eta_s.current         <-   eta_s[1]      
theta.current         <-   theta[1,]
w.current             <-   w[1,]
sv.current            <-   sv[1,]
v.current             <-   v[1,]
u.current             <-   u[1,]
gamma.current         <-   gamma[1,]
sigma2.current        <-   sigma2[1,]


################################################################################
################################################################################
##### ***START OF MCMC LOOP*** #################################################
################################################################################
################################################################################
for(i in 2:nLong) {
    print(i)
    if(i%%100==0){ print(i) }
    if(i%%500==0){ tracePlots() }
    if(i%%10000==0 | i==nLong){ burnt <- (nBurn+1):(i/nThin); printDeviance() }
    ## Tune the proposal variances every 200 iterations in the first 5000 iterations
    if(i%%200==0 & i <= 5000) {
        phi_s.prop.sd       <- phi_s.prop.sd   *adaptJump(6,V.acc/200)          # Tune the proposal SD for log variance for normal prior for superregion random intercepts (log kappa_a^r)
        eta_s.prop.sd       <- eta_s.prop.sd   *adaptJump(6,V.acc/200)          # Tune the proposal SD for log variance for normal prior for superregion random slopes (log kappa_b^r)
        phi_r.prop.sd       <- phi_r.prop.sd   *adaptJump(6,V.acc/200)          # Tune the proposal SD for log variance for normal prior for region random intercepts (log kappa_a^s)
        eta_r.prop.sd       <- eta_r.prop.sd   *adaptJump(6,V.acc/200)          # Tune the proposal SD for log variance for normal prior for region random slopes (log kappa_b^s)
        phi_c.prop.sd       <- phi_c.prop.sd   *adaptJump(6,V.acc/200)          # Tune the proposal SD for log variance for normal prior for country random intercepts (log kappa_a^c)
        eta_c.prop.sd       <- eta_c.prop.sd   *adaptJump(6,V.acc/200)          # Tune the proposal SD for log variance for normal prior for country random slopes (log kappa_b^c)
        theta_c.prop.sd     <- theta_c.prop.sd   *adaptJump(1,theta_c.acc/200)  # Tune the proposal SD for log precision parameter for random walk at country level (log lambda_c)
        theta_r.prop.sd     <- theta_r.prop.sd   *adaptJump(1,theta_r.acc/200)  # Tune the proposal SD for log precision parameter for random walk at region level (log lambda_s)
        theta_s.prop.sd     <- theta_s.prop.sd   *adaptJump(1,theta_s.acc/200)  # Tune the proposal SD for log precision parameter for random walk at superregion level (log lambda_r)
        theta_g.prop.sd     <- theta_g.prop.sd   *adaptJump(1,theta_g.acc/200)  # Tune the proposal SD for log precision parameter for random walk at global level (log lambda_g)
        tau.prop.sd         <- tau.prop.sd      *adaptJump(1,tau.acc/200)       # Tune the proposal SD for log variance for within-study errors that differ between age groups (log tau)
        phi_natl.prop.sd    <- phi_natl.prop.sd   *adaptJump(3,phi.acc/200)     # Tune the proposal SD for log variance of random effects for national studies (log nu_"national")
        phi_subn.prop.sd    <- phi_subn.prop.sd   *adaptJump(3,phi.acc/200)     # Tune the proposal SD for log variance of random effects for subnational studies (log nu_"su")
        phi_comm.prop.sd    <- phi_comm.prop.sd   *adaptJump(3,phi.acc/200)     # Tune the proposal SD for log variance of random effects for community studies (log nu_"c")
        for(j in 1:ng) {                                                        # Tune proposal variances for log variances for country-specific random spline coefficients
            log.sigma2.prop.sd[j] <- log.sigma2.prop.sd[j]*adaptJump(1,sigma2.acc[j]/200) 
            } 
        V.acc <- theta_c.acc <- theta_r.acc <- theta_s.acc <- theta_g.acc <- tau.acc <- phi.acc <- 0 # Set the acceptance countS to zero
        sigma2.acc          <- rep(0, ng)                                       # Set the acceptance counts for the country-specific random spline coefficients to zero
    }
    
    #### STUDY-SPECIFIC RANDOM EFFECTS: update variances #######################
    #### see pp8-9 and 13 of Danaei et al ######################################
    ############################################################################
    ssre           <- theta.current[(2+2*L+2*sum(multipleRegionsInSregion)+2*J+1):(2+2*L+2*sum(multipleRegionsInSregion)+2*J+N)] # Extract the current values from the theta matrix
    phi_natl.star  <- rnorm(1, phi_natl.current, phi_natl.prop.sd)                   # Propose new value for log variance of national study-specific random effects
    phi_subn.star  <- rnorm(1, phi_subn.current, phi_subn.prop.sd)                   # Propose new value for log variance of subnational study-specific random effects
    phi_comm.star  <- rnorm(1, phi_comm.current, phi_comm.prop.sd)                   # Propose new value for log variance of community study-specific random effects
    if(phi_natl.star < phi_subn.star & phi_subn.star < phi_comm.star){               # Constraint: national level has smaller variance than subnational and in turn community
        R <- exp(ssreLik(N_natl, phi_natl.star, ssre[uid.match$coverage=="National"]) + LogPriorPhi(phi_natl.star) 
                 - ssreLik(N_natl, phi_natl.current, ssre[uid.match$coverage=="National"]) - LogPriorPhi(phi_natl.current)
                 + ssreLik(N_subn, phi_subn.star, ssre[uid.match$coverage=="Subnational"]) + LogPriorPhi(phi_subn.star) 
                 - ssreLik(N_subn, phi_subn.current, ssre[uid.match$coverage=="Subnational"]) - LogPriorPhi(phi_subn.current)
                 + ssreLik(N_comm, phi_comm.star, ssre[uid.match$coverage=="Community"]) + LogPriorPhi(phi_comm.star) 
                 - ssreLik(N_comm, phi_comm.current, ssre[uid.match$coverage=="Community"]) - LogPriorPhi(phi_comm.current)) # Acceptance ratio R for proposed state
        if(runif(1) < R){                                                            # Metropolis-Hastings update: accept proposed state with probability R
            phi_natl.current   <- phi_natl.star
            phi_subn.current   <- phi_subn.star
            phi_comm.current   <- phi_comm.star
            phi.acc            <- phi.acc + 1
            V.ssre[uid.match$coverage == "National"]    <- exp(phi_natl.current)
            V.ssre[uid.match$coverage == "Subnational"] <- exp(phi_subn.current)
            V.ssre[uid.match$coverage == "Community"]   <- exp(phi_comm.current)
            V.diag[(2+2*L+2*sum(multipleRegionsInSregion)+2*J+1):(2+2*L+2*sum(multipleRegionsInSregion)+2*J+N)] <- V.ssre   
            VInv.diag <- 1/V.diag }   }
    
    #### UPDATE LINEAR RANDOM INTERCEPTS AND SLOPES VARIANCES AND THETA MATRIX #
    #### See pp5 and 13-14 of Danaei et al #####################################
    ############################################################################
    phi_s.sd.star <- rnorm(1, sqrt(exp(phi_s.current)), phi_s.prop.sd)               # Propose SD for normal prior for superregion random intercepts
    eta_s.sd.star <- rnorm(1, sqrt(exp(eta_s.current)), eta_s.prop.sd)               # Propose SD for normal prior for superregion random slopes
    phi_r.sd.star <- rnorm(1, sqrt(exp(phi_r.current)), phi_r.prop.sd)               # Propose SD for normal prior for region random intercepts
    eta_r.sd.star <- rnorm(1, sqrt(exp(eta_r.current)), eta_r.prop.sd)               # Propose SD for normal prior for region random intercepts
    phi_c.sd.star <- rnorm(1, sqrt(exp(phi_c.current)), phi_c.prop.sd)               # Propose SD for normal prior for country random intercepts
    eta_c.sd.star <- rnorm(1, sqrt(exp(eta_c.current)), eta_c.prop.sd)               # Propose SD for normal prior for country random slopes

    ##### if any sd's are <=0, reject #############################
    if(phi_s.sd.star > 0 & eta_s.sd.star > 0 & phi_r.sd.star > 0 & eta_r.sd.star > 0 & phi_c.sd.star > 0 & eta_c.sd.star > 0) {
        V.diag.star <- c( 1/epsilon, 
                          1/epsilon, 
                          rep(phi_s.sd.star^2, L),     
                          rep(eta_s.sd.star^2, L), 
                          rep(phi_r.sd.star^2, sum(multipleRegionsInSregion)),   
                          rep(eta_r.sd.star^2, sum(multipleRegionsInSregion)), 
                          rep(phi_c.sd.star^2, J), 
                          rep(eta_c.sd.star^2, J),   
                          V.ssre, 
                          rep(1/epsilon,p)   )                                       # Proposed diagonal of variance matrix         
        VInv.diag.star <- 1/V.diag.star 
        ##### Gibbs step to propose new theta based on the new variances #######
        Q <- Q.star <- F.spam.prime %*% (SigmaInv.diag * F.spam)
        VinvM <- F.spam.prime %*% (SigmaInv.diag * (y - (u.current[which.countryTime] + v.current[which.regionTime] 
                                                         + sv.current[which.sregionTime] + w.current[which.time]) - varphiPlusC.age))
        diag(Q.star) <- diag(Q.star) + VInv.diag.star
        U.star <- update.spam.chol.NgPeyton(U.theta.init, Q.star)
        M.star <- backsolve(U.star, forwardsolve(U.star, VinvM))
        theta.star <- M.star + backsolve(U.star, rnorm(length(VinvM)))

        ##### Calculate the equivalent information for the existing state ######
        diag(Q) <- diag(Q) + VInv.diag
        U <- update.spam.chol.NgPeyton(U.theta.init, Q)
        M <- backsolve(U, forwardsolve(U, VinvM))
        F.theta.star.Mm <- F%*%theta.star + u.current[which.countryTime] + v.current[which.regionTime] + sv.current[which.sregionTime] + w.current[which.time]
        R <- exp((.5*(sum(log(SigmaInv.diag)) -            
                          sum(SigmaInv.diag*(y - F.theta.star.Mm - R.age.spam %*% gamma.current)^2))) +      
                     .5*(sum(log(VInv.diag.star)) -              
                             sum(VInv.diag.star * theta.star^2)) - sum(log(diag(U.star))) + 
                     .5 * t(theta.star - M.star) %*% Q.star %*% (theta.star - M.star) -
                     (.5*(sum(log(SigmaInv.diag)) - sum(SigmaInv.diag*(y - F.theta.Mm - R.age.spam %*% gamma.current)^2))) -
                     .5*(sum(log(VInv.diag)) - sum(VInv.diag* theta.current^2)) + sum(log(diag(U))) -
                     .5 *t(theta.current - M) %*% Q %*% (theta.current - M))               
        ##### Accept/reject. Note: Gelman prior is flat on SD scale #####
        if(runif(1) < R) {                                                           # Metropolis-Hastings update: accept proposed state with probability R
            phi_s.current <- log(phi_s.sd.star^2)
            eta_s.current <- log(eta_s.sd.star^2)
            phi_r.current <- log(phi_r.sd.star^2)
            eta_r.current <- log(eta_r.sd.star^2)
            phi_c.current <- log(phi_c.sd.star^2)
            eta_c.current <- log(eta_c.sd.star^2)
            theta.current <- theta.star
            VInv.diag <- VInv.diag.star
            V.diag <- V.diag.star
            V.acc <- V.acc+1
            F.theta.Mm <- as.vector(F.spam %*% theta.current + u.current[which.countryTime] + v.current[which.regionTime] + sv.current[which.sregionTime] + w.current[which.time])
        } }
    
    #### NON-LINEAR TRENDS #####################################################
    #### Users should refer to pp5-7 and pp14-16 of Danaei et al for the line- #
    #### by-line details of this section of code ###############################
    ############################################################################
    # Component of nonlinear trend at national level #
    theta_c.star <- rnorm(1, theta_c.current, theta_c.prop.sd)                              # Propose theta_c*
    if(theta_c.star < theta_r.current & theta_c.star < theta.max) {                         # Only enter update step if constraints are satisfied
        u.Prec.yMinusMean     <- SigmaInv.diag * (y - F %*% theta.current - (v.current[which.regionTime] + sv.current[which.sregionTime] + w.current[which.time]) - varphiPlusC.age) 
        agg1                  <- aggregate(u.Prec.yMinusMean,list(which.countryTime),sum)
        agg2                  <- aggregate(SigmaInv.diag,list(which.countryTime),sum)
        Vinv.M.u[agg1[,1]]    <- agg1[,2]
        V1uInv.diag[agg2[,1]] <- agg2[,2]
        uStar <- lapply(1:J, u.prop.function, tPtsPerCountry, V1uInv.diag, Vinv.M.u, theta_c.star, theta_c.current, u.current)
        u.star <- rep(NA, J*T)
        u.dens.old <- u.dens.star <- rep(NA, J)
        for(j in 1:J){
            u.star[((j-1)*T+1):(j*T)] <- uStar[[j]]$u.star
            u.dens.old[j]             <- uStar[[j]]$dens.old
            u.dens.star[j]            <- uStar[[j]]$dens.star }
        u.star          <- Re(u.star)           
        u.dens.old      <- Re(u.dens.old)
        u.dens.star     <- Re(u.dens.star)
        F.theta.Mm.star <- as.vector(F.spam %*% theta.current + u.star[which.countryTime] + v.current[which.regionTime] + sv.current[which.sregionTime] + w.current[which.time]) 
        R.age.star      <- R.age
        R.age.star      <- as.spam(R.age.star)
        R <- exp(LogLik(SigmaInv.diag, gamma.current, R.age.spam, F.theta.Mm.star) -        # Acceptance ratio R for proposed state
                     LogLik(SigmaInv.diag, gamma.current, R.age.spam, F.theta.Mm) + 
                     LogPostThetaC(theta_c.star, u.star) -                   
                     LogPostThetaC(theta_c.current, u.current)  +  
                     sum(u.dens.old) -                     
                     sum(u.dens.star)   )
        if(runif(1) < R){                                                                   # Metropolis-Hastings step: accept proposed state with probability R
            theta_c.current <- theta_c.star 
            u.current       <- u.star
            F.theta.Mm      <- F.theta.Mm.star
            theta_c.acc     <- theta_c.acc+1 
            } 
        }
    
    ## Component of nonlinear trend at regional level ##########################
    theta_r.star <- rnorm(1, theta_r.current, theta_r.prop.sd)  
    if(theta_r.star > theta_c.current & theta_r.star < theta_s.current & theta_r.star < theta.max) {
        v.Prec.yMinusMean     <- SigmaInv.diag * (y - F %*% theta.current - (u.current[which.countryTime] + sv.current[which.sregionTime] + w.current[which.time]) - varphiPlusC.age) 
        agg3                  <- aggregate(v.Prec.yMinusMean,list(which.regionTime),sum)
        agg4                  <- aggregate(SigmaInv.diag,list(which.regionTime),sum)
        Vinv.M.v[agg3[,1]]    <- agg3[,2]
        V1vInv.diag[agg4[,1]] <- agg4[,2]
        u.star                <- rep(NA, K*T)
        u.dens.old <- u.dens.star <- rep(NA, K)
        uStar <- lapply(1:K, u.prop.function, tPtsPerRegion, V1vInv.diag, Vinv.M.v, theta_r.star, theta_r.current, v.current)
        for(j in 1:K){
            u.star[((j-1)*T+1):(j*T)] <- uStar[[j]]$u.star
            u.dens.old[j]             <- uStar[[j]]$dens.old
            u.dens.star[j]            <- uStar[[j]]$dens.star }
        u.star         <- Re(u.star)
        u.dens.old     <- Re(u.dens.old)
        u.dens.star    <- Re(u.dens.star)
        # set to 0 the v's in those regions that are also super-regions
        u.star[rep(!multipleRegionsInSregion, each=T)] <- 0      
        u.dens.star[!multipleRegionsInSregion]         <- 0       
        u.dens.old[!multipleRegionsInSregion]          <- 0      
        F.theta.Mm.star <- as.vector(F.spam %*% theta.current + u.current[which.countryTime] + u.star[which.regionTime] + sv.current[which.sregionTime] + w.current[which.time]) 
        R.age.star      <- R.age
        R.age.star      <- as.spam(R.age.star)
        R <- exp(LogLik(SigmaInv.diag, gamma.current, R.age.spam, F.theta.Mm.star) -       # Acceptance ratio R for proposed state
                     LogLik(SigmaInv.diag, gamma.current, R.age.spam, F.theta.Mm) + 
                     LogPostThetaR(theta_r.star, u.star) - 
                     LogPostThetaR(theta_r.current, v.current)  + 
                     sum(u.dens.old) - 
                     sum(u.dens.star)   )
        if(runif(1) < R) {                                                                 # Metropolis-Hastings step: accept proposed state with probability R
            theta_r.current <- theta_r.star 
            v.current       <- u.star
            F.theta.Mm      <- F.theta.Mm.star
            theta_r.acc     <- theta_r.acc+1 
            } 
        } 
    
    ## Component of nonlinear trend at superregional level #####################
    theta_s.star <- rnorm(1, theta_s.current, theta_s.prop.sd)  
    if(theta_s.star > theta_r.current & theta_s.star < theta_g.current & theta_s.star < theta.max) {
        sv.Prec.yMinusMean     <- SigmaInv.diag  * (y - F %*% theta.current - (u.current[which.countryTime] + v.current[which.regionTime] + w.current[which.time]) - varphiPlusC.age) 
        agg5                   <- aggregate(sv.Prec.yMinusMean,list(which.sregionTime),sum)
        agg6                   <- aggregate(SigmaInv.diag,list(which.sregionTime),sum)
        Vinv.M.sv[agg5[,1]]    <- agg5[,2]
        V1svInv.diag[agg6[,1]] <- agg6[,2]   
        u.star          <- rep(NA, L*T)
        u.dens.old      <- u.dens.star <- rep(NA, L)
        uStar <- lapply(1:L, u.prop.function, tPtsPerSregion, V1svInv.diag, Vinv.M.sv, theta_s.star, theta_s.current, sv.current)
        for(j in 1:L){
            u.star[((j-1)*T+1):(j*T)] <- uStar[[j]]$u.star
            u.dens.old[j]             <- uStar[[j]]$dens.old
            u.dens.star[j]            <- uStar[[j]]$dens.star
        }
        u.star          <- Re(u.star)
        u.dens.old      <- Re(u.dens.old)
        u.dens.star     <- Re(u.dens.star)
        F.theta.Mm.star <- as.vector(F.spam %*% theta.current + u.current[which.countryTime] + v.current[which.regionTime] + u.star[which.sregionTime] + w.current[which.time]) 
        R.age.star      <- R.age
        R.age.star      <- as.spam(R.age.star)
        R <- exp(LogLik(SigmaInv.diag, gamma.current, R.age.spam, F.theta.Mm.star) -        # Acceptance ratio R for proposed state
                     LogLik(SigmaInv.diag, gamma.current, R.age.spam, F.theta.Mm) + 
                     LogPostThetaS(theta_s.star, u.star) -  
                     LogPostThetaS(theta_s.current, sv.current) + 
                     sum(u.dens.old) - 
                     sum(u.dens.star))
        if(runif(1) < R) {                                                                  # Metropolis-Hastings step: accept proposed state with probability R
            theta_s.current <- theta_s.star 
            sv.current      <- u.star
            F.theta.Mm      <- F.theta.Mm.star
            theta_s.acc     <- theta_s.acc+1 
            }   
        }
    
    ## Component of nonlinear trend at global level ############################
    theta_g.star <- rnorm(1, theta_g.current, theta_g.prop.sd)  
    if(theta_g.star > theta_s.current & theta_g.star < theta.max) { 
        w.Prec.yMinusMean  <- SigmaInv.diag * (y - F %*% theta.current - (u.current[which.countryTime] + sv.current[which.sregionTime] + v.current[which.regionTime]) - varphiPlusC.age) 
        for(j in unique(which.time)){  
            Vinv.M.w[j]    <- sum(w.Prec.yMinusMean[which.time==j]) 
            V1wInv.diag[j] <- sum(SigmaInv.diag[which.time==j])
        }
        uStar              <- lapply(1, u.prop.function, 2, V1wInv.diag, Vinv.M.w, theta_g.star, theta_g.current, w.current)
        u.star             <- uStar[[1]]$u.star
        u.dens.old         <- uStar[[1]]$dens.old
        u.dens.star        <- uStar[[1]]$dens.star
        u.star             <- as.vector(Re(u.star))
        u.dens.old         <- Re(u.dens.old)
        u.dens.star        <- Re(u.dens.star)
        F.theta.Mm.star    <- as.vector(F.spam %*% theta.current + u.current[which.countryTime] + v.current[which.regionTime] + sv.current[which.sregionTime] + u.star[which.time]) 
        R.age.star         <- R.age
        R.age.star         <- as.spam(R.age.star)
        R <- exp(LogLik(SigmaInv.diag, gamma.current, R.age.spam, F.theta.Mm.star) -        # Acceptance ratio R for proposed state
                     LogLik(SigmaInv.diag, gamma.current, R.age.spam, F.theta.Mm) + 
                     LogPostThetaG(theta_g.star, u.star) - 
                     LogPostThetaG(theta_g.current, w.current)  + 
                     u.dens.old - 
                     u.dens.star)
        if(runif(1) < R) {                                                                  # Metropolis-Hastings step: accept proposed state with probability R
            theta_g.current <- theta_g.star 
            w.current       <- u.star
            F.theta.Mm      <- F.theta.Mm.star
            theta_g.acc     <- theta_g.acc+1 
            } 
        }
    
    ##### GAMMA: update using Gibbs ############################################
    ############################################################################
    Q       <- R.age.spam.prime %*% (SigmaInv.diag * R.age.spam)                            # Full conditional precision of gamma
    diag(Q) <- diag(Q) + WInv.diag                                                          # Full conditional precision of gamma
    U       <- update.spam.chol.NgPeyton(U.gamma.init, Q)                                   # Update Cholesky decomposition
    VinvM   <- as.matrix(R.age.spam.prime) %*% (SigmaInv.diag * (y - F.theta.Mm))           # Full conditional precision x conditional mean
    gamma.current   <- backsolve(U, forwardsolve(U, VinvM, trans=T) + rnorm(length(VinvM))) # Updated gamma after Gibbs step
    varphiPlusC.age <- rowSums((rep(1, I) %*% t(gamma.current[1:ng]) +  matrix(gamma.current[-(1:ng)], J, ng)[which.country,]) * ageMat) # Update derived vector for future calculations
    
    ##### SIGMA2: update using Metropolis-Hastings #############################
    ############################################################################
    for(j in 1:ng) { # Loop over the ng parameters
        log.sigma2.star <- rnorm(1, log(sigma2.current[j]), log.sigma2.prop.sd[j])          # Propose sigma2* values on log scale
        R <- exp(-.5*(J*log.sigma2.star + 1/exp(log.sigma2.star) * sum(gamma.current[ng + (((j-1)*J+1):(j*J))]^2)) + LogPriorPhi(log.sigma2.star) +
                     .5*(J*log(sigma2.current[j]) + 
                         1/sigma2.current[j] * sum(gamma.current[ng+(((j-1)*J+1):(j*J))]^2)) - LogPriorPhi(log(sigma2.current[j]))) # Posterior probability R for proposed state
        if(runif(1) < R) {                                                                  # Metropolis-Hastings update: accept proposed state with probability R
            sigma2.current[j]  <- exp(log.sigma2.star)
            WInv.diag[ng +(((j-1)*J+1):(j*J))] <- rep(1/exp(log.sigma2.star), J)
            sigma2.acc[j] <- sigma2.acc[j]+1 
            } 
        } 
    
    ##### TAU: update using Metropolis-Hastings ################################
    ############################################################################
    tau.star           <- rnorm(1, tau.current, tau.prop.sd)                                # Propose tau* value on log scale
    Sigma.diag.star    <- sem^2 + exp(tau.star)                                             # Variance diagonal including tau* value
    SigmaInv.diag.star <- 1/Sigma.diag.star                                                 # Inverse variance diagonal including tau* value
    R <- exp(LogLik(SigmaInv.diag.star, gamma.current, R.age.spam, F.theta.Mm) + LogPriorPhi(tau.star) -
                 LogLik(SigmaInv.diag, gamma.current, R.age.spam, F.theta.Mm) - LogPriorPhi(tau.current)) # Posterior probability R for proposed state
    if(runif(1) < R) {                                                                      # Metropolis-Hastings update: accept proposed state with probability R
        tau.current    <- tau.star
        tau.acc        <- tau.acc+1
        Sigma.diag     <- Sigma.diag.star
        SigmaInv.diag  <- SigmaInv.diag.star 
    } 
    
    ##### Save state every nThin iteration ######################################
    ############################################################################
    if(i%%nThin==0){
        j            <- i/nThin 
        theta_g[j]   <- theta_g.current 
        theta_s[j]   <- theta_s.current 
        theta_r[j]   <- theta_r.current 
        theta_c[j]   <- theta_c.current 
        phi_c[j]     <- phi_c.current   
        phi_r[j]     <- phi_r.current      
        phi_s[j]     <- phi_s.current  
        eta_c[j]     <- eta_c.current 
        eta_r[j]     <- eta_r.current
        eta_s[j]     <- eta_s.current
        phi_natl[j]  <- phi_natl.current 
        phi_subn[j]  <- phi_subn.current   
        phi_comm[j]  <- phi_comm.current
        tau[j]       <- tau.current 
        theta[j,]    <- theta.current   
        w[j,]        <- w.current        
        sv[j,]       <- sv.current         
        v[j,]        <- v.current      
        u[j,]        <- u.current  
        gamma[j,]    <- gamma.current 
        sigma2[j,]   <- sigma2.current 
        deviance[j]  <- LogLik(SigmaInv.diag, gamma.current, R.age, F.theta.Mm) * -2 + I * log(2*pi)   
        } 
    }
################################################################################
##### ***END OF MCMC LOOP*** ###################################################
################################################################################


##### Save the burnt-in, thinned chains ########################################
################################################################################
burnt      <- ((nBurn/nThin)+1):nIts
u          <- u[burnt,]
v          <- v[burnt,]
sv         <- sv[burnt,]
w          <- w[burnt,]
theta      <- theta[burnt,]
phi_natl   <- phi_natl[burnt]
phi_subn   <- phi_subn[burnt]
phi_comm   <- phi_comm[burnt]
tau        <- tau[burnt]
theta_g    <- theta_g[burnt]
theta_s    <- theta_s[burnt]
theta_r    <- theta_r[burnt]
theta_c    <- theta_c[burnt]
phi_c      <- phi_c[burnt]
phi_r      <- phi_r[burnt]
phi_s      <- phi_s[burnt]
eta_c      <- eta_c[burnt]
eta_r      <- eta_r[burnt]
eta_s      <- eta_s[burnt]
gamma      <- gamma[burnt,]
deviance   <- deviance[burnt]
save.image(paste(filename,"_Burnt.RData",sep='')); tracePlots()

